%% Fast Polynomial Approximation of Heat Diffusion on Manifolds
% This package is downloaded from http://www.stat.wisc.edu/~mchung/chevyshev

% This package performs Polynomial approximation of heat diffusion on
% surface meshes [1] and generate Figure 1 in [1] and compute the
% reconstruction error. The recurrence relation part is modified from the 
% Spectral Graph Wavelet Transform (SGWT) [2] toolbox (https://wiki.epfl.ch/sgwt).
%
%
% References:
%
% [1] Huang, S.-G., Lyu, I., Qiu, A., Chung, M.K.: 
% Fast polynomial approximation to heat diffusion in manifolds, 
% 2019 Medical Image Computing and Computer Assisted Intervention (MICCAI)
% http://pages.stat.wisc.edu/~mchung/papers/huang.2019.MICCAI.pdf
%
% [2] Tan, M., Qiu, A.: Spectral Laplace-Beltrami wavelets with applications 
% in medical images. IEEE Transactions on Medical Imaging 34, 1005-1017, 2015
%
% 
%
% (C) 2019 Shih-Gu Huang shihgu@gmail.com
%          Moo K. Chung  mkchung@wisc.edu
%          Universtiy of Wisconsin-Madison
%
%
%
% Update history:
%     Jul. 29, 2019 created Huang
%     Aug. 01, 2019 edited Chung
%     Sep. 05, 2019 modified Huang. Modifed to MATLAB surface mesh format 


%-------------------------------
%% Load hippocampus surface mesh. 
% The left hippocampus surface data (hippocampus.mat) is downloaded from 
% http://brainimaging.waisman.wisc.edu/~chung/lb

load('hippocampus.mat')	% left and right hippocampus surface
surf=hippoleft;             % use only the left surface

figure
subplot(141)
figure_patch(surf,[0.85 0.85 0.7],1);
view([185 15]); camlight left; title('Original');


%% Discretization of Laplace-Beltrami operator
addpath('./LB/')
% folder containing some functions modified from Spectral Laplace-Beltrami Wavelets toolbox [2] 
% toolbox for computing LB-operator
L=LB_operator(surf); 


%% Heat diffusion using polynomial approximation methods
% Heat diffusion of the left hippocampus surface mesh coordinates using the 
% Chebyshev, Hermite and Laguerre polynomial expansion method.

sigma=1.5;    % diffision time
m=100;        % degree of polynomail expansion
eigmax = eigs(L,1,'lm'); % maximun eigenvalue of Laplace-Beltrami operator

% Chebyshev polynomial expansion
surfC.faces=surf.faces;
[surfC.vertices,~]=heat_chebyshev(surf.vertices,L,sigma,m,eigmax);  
subplot(142); figure_patch(surfC,[0.85 0.85 0.7],1);
view([185 15]); camlight left; title('Chebyshev'); 

% Hermite polynomial expansion
surfH.faces=surf.faces;
[surfH.vertices,~]=heat_hermite(surf.vertices,L,sigma,m);  % Heat diffusion of surface mesh coordinates
subplot(143); figure_patch(surfH,[0.85 0.85 0.7],1);
view([185 15]); camlight left; title('Hermite');

% Laguerre polynomial expansion
surfL.faces=surf.facesl
[surfL.vertices,~]=heat_laguerre(surf.vertices,L,sigma,m);  % Heat diffusion of surface mesh coordinates
subplot(144); figure_patch(surfL,[0.85 0.85 0.7],1);
view([185 15]); camlight left; title('Laguerre'); 



%The reconstruction errror between the polynomial expansion method 
%and the original surface mesh measured in mean squared error (MSE) as 
%measured in voxel width squared.

%% MSE between the polynomial expansion method and the original surface mesh


mRange=[5:60];   % range of expansion degree m
original=hippoleft.vertices;  % original surface mesh coordinates

mse_chebyshev=[];
mse_laguerre=[];
mse_hermite=[];
for m=mRange
    
    [vC,~]=heat_chebyshev(surf.vertices,L,sigma,m,eigmax);  % Chebyshev polynomial expansion    
    [vH,~]=heat_hermite(surf.vertices,L,sigma,m);           % Hermite polynomial expansion    
    [vL,~]=heat_laguerre(surf.vertices,L,sigma,m);          % Laguerre polynomial expansion
    
    mseC=mean(sum((vC-original).^2,2));     % MSE of Chebyshev polynomial expansion method   
    mseH=mean(sum((vH-original).^2,2));     % MSE of Hermite polynomial expansion method    
    mseL=mean(sum((vL-original).^2,2));     % MSE of Laguerre polynomial expansion method
    
    mse_chebyshev=[mse_chebyshev mseC];    
    mse_hermite=[ mse_hermite mseH];   
    mse_laguerre=[mse_laguerre mseL];
        
end

figure
hold on; plot(mRange,mse_chebyshev,'-','Linewidth',1.5,'Color',[0, 0, 0.8])
hold on; plot(mRange,mse_hermite,'-.','Linewidth',1.5,'Color',[0, 0.6, 0])
hold on; plot(mRange,mse_laguerre,'--','Linewidth',1.5,'Color',[0.8, 0, 0])

xlim([0 55]); ylim([0 5]);
legend('Chebyshev','Hermite','Laguerre'); legend boxoff;
xlabel('Expansion degree \itm')
title('Mean squared error')




